"""
    Blackboard Analysis Tools
    Copyright 2013, Jeroen Doggen, jeroendoggen@gmail.com
"""

from __future__ import print_function, division  # We require Python 2.6+

import os
import shutil


class InputOutput():
    """ Timer to check the speed of the tool itself (benchmarking) """

    def __init__(self, input_path, output_path, logger, settings, reporter, analyser):
        self.input_path = input_path
        self.output_path = output_path
        self.logger = logger
        self.settings = settings
        self.reporter = reporter
        self.analyser = analyser

    def run(self):
        """ Run all the tests """
        self.create_student_folders()
        self.move_student_files()
        self.process_badly_named_files()

    def create_student_folders(self):
        """ Create the needed student folders """
        for student in self.analyser.studentnames_list:
            if not os.path.exists(self.settings.output_path + student):
                os.makedirs(self.settings.output_path + student)

    def move_student_files(self):
        """ Move assignment files to student folders (using one process)"""
        try:
            for inputfile in os.listdir(self.settings.output_path):
                if os.path.isfile(os.path.join(self.settings.output_path, inputfile)):
                    # TODO: use a list for this?
                    self.move_files(inputfile)
        except OSError:
            self.exit_program("moving student files to output folder")

    def move_files(self, inputfile):
        """ Move assignment file to the correct student folder """
        #print(inputfile)
        for student in self.analyser.studentnames_list:
            #print(student)
            if '.' in student:
                student = self.swap_string(student)
                if student in inputfile:
                    #print(student)
                    student = self.swap_string(student)
                    if os.path.exists(student):
                        shutil.copy2(inputfile, self.output_path + student)
                        #os.remove(inputfile)

    def process_badly_named_files(self):
        #"""
        #Scan for bad filenames (where student mail is not in the filename)
        #Copy these files to 'studentname' folder anyway
        #Add logging to provide some feedback
        #"""
        for txtfile in self.analyser.txt_files_list:
            studentname = self.analyser.get_studentname(txtfile)
            filename = self.analyser.get_filename(txtfile)
            if filename is not None:
                assignment = self.analyser.get_assignment(txtfile)
                self.reporter.bad_filenames_counter += 1
                self.reporter.bad_filenames += " - " + str(studentname) + ": "
                self.reporter.bad_filenames += str(filename) + " --> in assignment: " + assignment
                shutil.copy2(filename, self.output_path + studentname)

    def swap_string(self, string):
        """ Swap strings around '.' symbol (for email address processing)"""
        #TODO: used multiple times (move to 'tools' class?
        string0 = string.split(".")[0]
        string1 = string.split(".")[1]
        string = string1 + "." + string0
        return(string)

    def cleanup(self):
        """ Clean up the output folder by removing all 'temp files' """
        try:
            for inputfile in os.listdir(self.settings.output_path):
                if os.path.isfile(os.path.join(self.settings.output_path, inputfile)):
                    if self.analyser.is_not_analysistool_file(inputfile):
                        #print(inputfile)
                        os.remove(inputfile)
        except OSError:
            self.logger.exit_program("cleaning up folders")



"""
    Blackboard Analysis Tools
    Copyright 2013, Jeroen Doggen, jeroendoggen@gmail.com
"""

from __future__ import print_function, division  # We require Python 2.6+

import os

from email.utils import parseaddr

from blackboard_analysis_tools.logger import Logger


class Analyser():
    """ Timer to check the speed of the tool itself (benchmarking) """
    txt_files_list = []
    studentnames_list = []

    def __init__(self, input_path, output_path, logger, settings, reporter):
        self.input_path = input_path
        self.output_path = output_path
        self.logger = logger
        self.settings = settings
        self.reporter = reporter

    def generate_txt_files_list(self):
        """ Generate the list with the txt files """
        try:
            os.chdir(self.output_path)
            for inputfile in os.listdir(self.output_path):
                    if os.path.isfile(os.path.join(self.output_path, inputfile)):
                        if self.is_not_analysistool_file(inputfile):
                            self.reporter.txt_files_counter += 1
                            if inputfile.endswith(".txt"):
                                self.txt_files_list.append(inputfile)
            #print(self.txt_files_list)
        except OSError:
            self.exit_program("reading the .txt files")

    def analyse_txt_files(self):
        """ Analyse all the .txt files """
        self.generate_studentnames_list()
        #print(self.studentnames_list)
        self.studentnames_list.sort()
        #print(self.studentnames_list)
        self.detect_late_assignments()

    def generate_studentnames_list(self):
        """ Get the student name from a given txt file """
        for txtfile in self.txt_files_list:
            self.studentnames_list.append(self.get_studentname(txtfile))

    def get_studentname(self, txtfile):
        """ Get the student name from a given txt file """
        studentname = ""
        with open(txtfile, 'r') as inputfile:
            for line in inputfile:
                if self.settings.name_detection_string in line:
                    studentname = parseaddr(line)[0]
                    #print(email)
                    #print(studentname)
                    studentname = studentname[:-self.settings.email_domain_length]
                    #print(studentname)
                    studentname = self.swap_string(studentname)
                    #print(email)
            inputfile.close()
        return(studentname)

    def is_not_analysistool_file(self, inputfile):
        """ Detect is a file is a logfile, outputfile from this tool """
        if inputfile != self.settings.studentlist_filename_final:
            if inputfile != self.settings.logfile:
                if inputfile != self.settings.summary_file:
                    return(True)
        else:
            return(False)
    def swap_string(self, string):
        """ Swap strings around '.' symbol (for email address processing)"""
        #print(string)
        string0 = string.split(".")[0]
        #print(string0)
        string1 = string.split(".")[1]
        #print(string1)
        string = string1 + "." + string0
        #print(string)
        return(string)

    def detect_late_assignments(self):
        """ Detect is an assignment is handed in late from a given .txt file """
        for txtfile in self.txt_files_list:
            with open(txtfile, 'r') as inputfile:
                for line in inputfile:
                    if self.settings.assignment_late_detection_string in line:
                        self.reporter.late_assignment_counter += 1
                        print("Late file: ", end="")
                        print(inputfile.name)
                        #print(line)
                        line = line.lstrip(self.assignmentname_detection_string)
                        #line = line.rstrip("NM\n")
                        #print(line)
                        #TODO: this should return something and go to the report (was not available in previous version)

    def get_filename(self, txtfile):
        """ Get the assignment filename from a given txt file """
        with open(txtfile, 'r') as inputfile:
            for line in inputfile:
                if self.settings.filename_detection_string in line:
                    # detect bad bad filename (not containing student email)
                    if line.find(self.settings.filename_analysis_string) == -1:
                        line = line.lstrip('\t')
                        line = line.lstrip(self.settings.filename_detection_string)
                        line = line.lstrip(" ")
                        line = line.rstrip()
                        return(line)

    def get_assignment(self, txtfile):
        """ Get the assignment name from a given .txt file """
        with open(txtfile, 'r') as inputfile:
            for line in inputfile:
                if self.settings.assignmentname_detection_string in line:
                    line = line.lstrip(self.settings.assignmentname_detection_string)
                    #line = line.rstrip("NM\n")
                    return(line)



"""
    Blackboard Analysis Tools
    Copyright 2013, Jeroen Doggen, jeroendoggen@gmail.com
"""

from __future__ import print_function, division  # We require Python 2.6+

import os


class Reporter():
    """ Logging class """
    txt_files_counter = 0
    late_assignment_counter = 0
    student_counter = 0
    bad_filenames_counter = 0
    assignment_counter = 0
    bad_filenames = ""

    def __init__(self, settings):
        self.settings = settings
        #self.analyser = analyser

    def write_statistics(self, studentnames_list):
        """ Write statistics to files """
        self.write_student_list(studentnames_list)
        self.write_summary()

    def write_student_list(self, studentnames_list):
        """ Write a list with the name of all students to a file """
        outfile = open(self.settings.studentlist_filename_temp, 'w+')
        # TODO: does not work because of cyclic arguments
        for student in studentnames_list:
            outfile.write(student + "\n")
        outfile.close()
        # TODO: probably works, but uses an empty input file
        self.remove_duplicate_students()

    def remove_duplicate_students(self):
        """ Remove duplicate students in the students_list """
        """ TODO: does not work? """
        lines_seen = set()  # holds lines already seen
        outfile = open(self.settings.studentlist_filename_final, "w+")
        for line in open(self.settings.studentlist_filename_temp, "r+"):
            if line not in lines_seen:  # not a duplicate
                outfile.write(line)
                lines_seen.add(line)
                self.student_counter += 1
        outfile.close()

    def write_summary(self):
        """ Write a summary of the analysis process to a logfile """
        try:
            os.chdir(self.settings.output_path)
            outfile = open(self.settings.summary_file, 'w+')
            outfile.write("Build summary:\n")
            outfile.write("--------------\n")
            outfile.write(" Total students: ")
            outfile.write(str(self.student_counter))
            outfile.write("\n Total assignments: ")
            outfile.write(str(self.txt_files_counter))
            outfile.write("\n Total files: ")
            outfile.write(str(self.txt_files_counter))
            outfile.write("\n Late files: ")
            outfile.write(str(self.late_assignment_counter))
            outfile.write("\n Bad filesnames: \n")
            outfile.write(str(self.bad_filenames))
            outfile.close()
            with open(self.settings.summary_file) as f:
                content = f.read()
                print(content)
            outfile.close()
        except OSError:
            self.exit_program("writing the summary")



"""
    Blackboard Analysis Tools
    Copyright 2013, Jeroen Doggen, jeroendoggen@gmail.com
"""

# TODO: report all students that hand in the assignment after the deadline
# TODO: ...

import os


class Settings:
    """ Contains all the tools to analyse Blackboard assignments """
    logfile = "blackboard_analysis_tools.log"
    script_path = os.getcwd()
    input_path = script_path + "/input/"
    output_path = script_path + "/output/"
    name_detection_string = "Naam:"
    email_domain_length = 5
    filename_detection_string = "Bestandsnaam:"
    assignmentname_detection_string = "Opdracht:"
    filename_analysis_string = "s.ap"
    assignment_late_detection_string = "juni"

    logfile = "blackboard_analysis_tools.log"
    studentlist_filename_temp = "studentlist_all.txt"
    studentlist_filename_final = "studentlist_final.txt"
    summary_file = 'summary.txt'

    def __init__(self):
        pass



"""
    Blackboard Analysis Tools
    Copyright 2013, Jeroen Doggen, jeroendoggen@gmail.com
"""

from __future__ import print_function, division  # We require Python 2.6+

import logging
import sys


class Logger():
    """ Logging class """
    logger = 0

    def __init__(self, logfile):
        self.set_logfile(logfile)
        self.info("Starting 'analysis tool': ")

    def set_logfile(self, logfile):
        """Set the logfile: for error & info messages"""
        try:
            self.logfile = logfile
            logging.basicConfig(filename=self.logfile,
                                level=logging.DEBUG,
                                format="%(asctime)s %(name)s %(levelname)s %(message)s")
            self.logger = logging.getLogger(__name__)
        except IOError:
            self.exit_program("opening the logfile (do you have write permission?)")

    def exit_program(self, message):
        """ Exit the program with a message
           TODO: this should move somewhere else (needed in multiple places)
        """
        print("Error while " + message)
        print("Closing application")
        sys.exit()

    def info(self, message):
        self.logger.info(message)



"""
    Blackboard Analysis Tools
    Copyright 2013, Jeroen Doggen, jeroendoggen@gmail.com
"""

# TODO: report all students that hand in the assignment after the deadline
# TODO: ...


from __future__ import print_function, division  # We require Python 2.6+

#import logging
import os
#import zipfile
#import shutil
#import sys
#import datetime
#import time
#import StringIO
#from multiprocessing import Process

#try:
    #from Queue import Queue
#except ImportError:
    #from queue import Queue

#from email.utils import parseaddr

from blackboard_analysis_tools.settings import Settings
from blackboard_analysis_tools.timing_analyser import TimingAnalyser
from blackboard_analysis_tools.logger import Logger
from blackboard_analysis_tools.unzipper import Unzipper
from blackboard_analysis_tools.analyser import Analyser
from blackboard_analysis_tools.reporter import Reporter
from blackboard_analysis_tools.input_output import InputOutput


class BlackboardAnalysisTools:
    """ Contains all the tools to analyse Blackboard assignments """

    def __init__(self):
        self.settings = Settings()
        self.timing_analyser = TimingAnalyser()
        self.mylogger = Logger(self.settings.logfile)
        self.reporter = Reporter(self.settings)
        self.unzipper = Unzipper(self.settings.input_path, self.settings.output_path, self.mylogger, self.reporter)
        self.analyser = Analyser(self.settings.input_path, self.settings.output_path, self.mylogger, self.settings, self.reporter)
        self.input_output = InputOutput(self.settings.input_path, self.settings.output_path, self.mylogger, self.settings, self.reporter, self.analyser)

    def run(self):
        #""" Run the program (call this from main) """
        self.unzipper.generate_zip_files_list()
        self.unzipper.unzip()
        self.analyser.generate_txt_files_list()
        self.analyser.analyse_txt_files()
        self.input_output.run()
        #TODO does not work because of cyclic passing of instances
        self.reporter.write_statistics(self.analyser.studentnames_list)
        self.input_output.cleanup()

    def exit_value(self):
        #"""TODO: Generate the exit value for the application."""
        #if (self.errors == 0):
        if (True):
            return 0
        else:
            return 42



"""
    Blackboard Analysis Tools
    Copyright 2013, Jeroen Doggen, jeroendoggen@gmail.com
"""

from __future__ import print_function, division  # We require Python 2.6+

import os
import zipfile

from blackboard_analysis_tools.logger import Logger


class Unzipper():
    """ Timer to check the speed of the tool itself (benchmarking) """
    zip_files_list = []

    def __init__(self, input_path, output_path, logger, reporter):
        self.input_path = input_path
        self.output_path = output_path
        self.logger = logger
        self.reporter = reporter

    def generate_zip_files_list(self):
        """ Generate the list with the zip files """
        try:
            os.chdir(self.input_path)
            for inputfile in os.listdir(self.input_path):
                if os.path.isfile(os.path.join(self.input_path, inputfile)):
                    if inputfile.endswith(".zip"):
                        self.zip_files_list.append(inputfile)
                        self.reporter.assignment_counter += 1
        except OSError:
            self.logger.exit_program("reading the .zip files (does the output folder exist?)")

    def unzip(self):
        """ Unzip all the .zip assignment files """
        print("Number of .zip files: ", end="")
        counter = 0
        for index, current_file in enumerate(self.zip_files_list):
            counter += 1
        print(counter)
        print("")
        for index, current_file in enumerate(self.zip_files_list):
            shortname = str(counter) + ".zip"
            self.unzip_onefile(current_file, shortname)
            counter += 1

    def unzip_onefile(self, current_file, shortname):
        """ Unzip one file """
        os.rename(current_file, shortname)
        myzip = zipfile.ZipFile(shortname)
        myzip.extractall(self.output_path)
        myzip.close()
        os.rename(shortname, current_file)



"""
    Blackboard Analysis Tools
    Copyright 2013, Jeroen Doggen, jeroendoggen@gmail.com
"""

import sys

from blackboard_analysis_tools.blackboard import BlackboardAnalysisTools


def run():
    """Run the main program"""
    assignment_analyser = BlackboardAnalysisTools()
    #assignment_analyser.init()
    assignment_analyser.run()
    return(assignment_analyser.exit_value())


if __name__ == "__main__":
    sys.exit(run())



"""
    Blackboard Analysis Tools
    Copyright 2013, Jeroen Doggen, jeroendoggen@gmail.com
"""

from __future__ import print_function, division  # We require Python 2.6+

import datetime


class TimingAnalyser:
    """ Timer to check the speed of the tool itself (benchmarking) """
    starttime = 0
    lasttime = 0

    def __init__(self):
        self.starttime = datetime.datetime.now()
        self.lasttime = datetime.datetime.now()

    def timedebug(self, function, message):
        """Print the current runtime + a message to the terminal """
        ##TODO timing is off by one!
        now = datetime.datetime.now()
        print(message, end=" ")
        function()
        delta = now - self.lasttime
        delta = delta.total_seconds()
        print("this took: " + str(delta) + " seconds")
        self.lasttime = datetime.datetime.now()
